indexing
	description: "XML name including (resolved) namespace"
	aspects: immutable
	license: "http://www.opensource.org/licenses/mit-license.html"
	revision: "$Id: nxml_name.e,v 1.4 2001/07/14 00:19:45 farnaud Exp $"
	
class NXML_NAME

inherit
	HASHABLE
		redefine
			is_equal,
			out
		end

creation
	make,
	make_empty,
	make_element,
	make_inherit

feature {NONE} -- Creation

	make_empty is
			-- Empty name.
		do
			namespace := default_namespace
			name := default_name
		ensure
			name: name.count = 0
		end
		
	make_element (a_name: STRING) is
			-- Name with empty namespace.
		require
			name: valid_name (a_name)
		do
			make_empty
			name := a_name
		ensure
			name: name.is_equal (a_name)
		end
		
	make_inherit (a_parent: NXML_NAME; a_name: STRING) is
			-- Inherit namespace.
		require
			name: valid_name (a_name)
		do
			namespace := a_parent.namespace
			name := a_name
		ensure
			namespace: namespace.is_equal (a_parent.namespace)
			name: name.is_equal (a_name)
		end
		
	make (a_namespace: NXML_NAMESPACE; a_name: STRING) is
			-- Make with namespace.
		require
			namespace: a_namespace /= Void
			name: valid_name (a_name)
		do
			namespace := a_namespace
			name := a_name
		ensure
			namespace: namespace.is_equal (a_namespace)
			name: name.is_equal (a_name)
		end
		
feature -- Access

	valid_name (a_name: STRING): BOOLEAN is
			-- Valid name?
		do
			Result := a_name /= Void 
					and then (a_name.count > 0)
		end

	prefixed (a_ns_map: DS_SPARSE_TABLE [STRING, NXML_NAMESPACE]): STRING is
			-- Output as XML (prefixed).
		require
			has_namespace: a_ns_map.has (namespace)
		do	
			!! Result.make (0)
			prefixed_in (Result, a_ns_map)
		ensure
			new_string: Result /= Void
			size: Result.count = (namespace.as_xml (a_ns_map).count + 1 + name.count)
		end
		
	prefixed_in (a_string: STRING; a_ns_map: DS_SPARSE_TABLE [STRING, NXML_NAMESPACE]) is
			-- Append prefixed name to string.
		require
			has_namespace: a_ns_map.has (namespace)
		do
			a_string.append_string (namespace.as_xml (a_ns_map))
			a_string.append_character (':')
			a_string.append_string (name)
		ensure
			as_xml_in: a_string.count = (old (a_string.count) + prefixed(a_ns_map).count)
		end

feature -- Twin

	with_name (a_name: STRING): NXML_NAME is
			-- New name with same namespace as current.
		require
			name: valid_name (a_name)	
		do
			!! Result.make_inherit (Current, a_name)
		ensure
			namespace: Result.namespace.is_equal (Current.namespace)
			name: Result.name.is_equal (a_name)
		end

	same_namespace (an_other: NXML_NAME): BOOLEAN is
			-- Same namespace as other?
		require
			not_void: an_other /= Void
		do
			Result := namespace.is_equal (an_other.namespace)
		ensure
			def: Result = namespace.is_equal (an_other.namespace)
		end
		
feature {NONE} -- Defaults

	default_namespace: NXML_NAMESPACE is
			-- Shared default namespace.
		once
			!! Result.make (default_name)
		end

	default_name: STRING is
			-- Shared empty string.
		once
			!! Result.make (0)
		end

feature -- Access

	namespace: NXML_NAMESPACE
			-- Namespace.

	name: STRING
			-- Name of element or attribute.

feature -- ANY

	is_equal (an_other: like Current): BOOLEAN is
			-- Equality.
		do
			Result := an_other = Current 
				or else (
						namespace.is_equal (an_other.namespace)
						and
						name.is_equal (an_other.name)
						)
		end

	out: STRING is
			-- Printable representation.
			-- namespace-URI':'name (not valid XML because resolved!)
		do
			!! Result.make (0)
			Result.append_string (namespace.out)
			Result.append_character (':')
			Result.append_string (name)
		end
		
feature -- Hashable

	hash_code: INTEGER is
			-- Hash code.
		do
			Result := (namespace.hash_code // 3) + (name.hash_code // 3)
			-- divide to avoid overflow, 3 is prime.
		end

invariant
	name_not_void: name /= Void
	namespace_not_void: namespace /= Void

end
